<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\Exceptions\MissingStartupException;
use Infomaker\Everyware\Base\Interfaces\NetworkStartup;
use Infomaker\Everyware\Base\Interfaces\ThemeStartup;
use Infomaker\Everyware\Support\NewRelicLog;

/**
 * Class ProjectStartup
 * @package Infomaker\Everyware\Base
 */
class ProjectStartup
{
    /**
     * @var self
     */
    protected static $instance;

    /**
     * @var ThemeStartup
     */
    private $themeStartup;

    /**
     * @var NetworkStartup
     */
    private $networkStartup;

    /**
     * @throws MissingStartupException
     */
    private function runNetworkStartup(): void
    {
        if ( ! $this->networkStartup) {
            throw new MissingStartupException('A NetworkStartup has not been registered');
        }

        $this->networkStartup->bootstrap();
    }

    /**
     * @throws MissingStartupException
     */
    private function runThemeStartup(): void
    {
        if ( ! $this->themeStartup) {
            throw new MissingStartupException('A ThemeStartup has not been registered');
        }

        $this->themeStartup->bootstrap();
    }


    private function setNetworkStartup(NetworkStartup $startup): ProjectStartup
    {
        $this->networkStartup = $startup;

        return $this;
    }

    private function setThemeStartup(ThemeStartup $startup): ProjectStartup
    {
        $this->themeStartup = $startup;

        return $this;
    }

    public static function registerNetworkStartup(NetworkStartup $startup): void
    {
        try {
            static::getInstance()
                ->setNetworkStartup($startup)
                ->runNetworkStartup();
        } catch (MissingStartupException $e) {
            NewRelicLog::error('Failed to run NetworkStartup', $e);
        }
    }

    public static function registerThemeStartup(ThemeStartup $startup): void
    {
        try {
            static::getInstance()
                ->setThemeStartup($startup)
                ->runThemeStartup();
        } catch (MissingStartupException $e) {
            NewRelicLog::error('Failed to run ThemeStartup', $e);
        }
    }

    public static function getInstance(): ProjectStartup
    {
        if (static::$instance === null) {
            static::$instance = new static();
        }

        return static::$instance;
    }
}
