<?php

namespace Spec\Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\Utilities;
use Infomaker\Everyware\Support\Environment;
use PhpSpec\ObjectBehavior;

class UtilitiesSpec extends ObjectBehavior
{
    private $base_theme_path = CUSTOM_TEMPLATE_DIRECTORY_URI;

    private $originalEnv;

    public function let(): void
    {
        $this->originalEnv = Environment::current();
    }

    public function letGo(): void
    {
        Environment::setCurrent($this->originalEnv);
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(Utilities::class);
    }

    public function it_can_fetch_json_using_the_template_uri_as_base(): void
    {
        $this->getCorrectJsUrl('json_file.json')->shouldReturn("$this->base_theme_path/assets/js/json_file.json");
    }

    public function it_will_return_a_correct_template_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');

        $this->getStaticTemplateUri()->shouldReturn(CUSTOM_TEMPLATE_DIRECTORY_URI);

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getStaticTemplateUri()->shouldReturn("$static_uri/" . CUSTOM_STATIC_TEMPLATE_DIRECTORY);
    }

    public function it_will_return_a_correct_custom_cdn_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');

        $this->getCdnUri(CUSTOM_STYLESHEET_DIRECTORY_URI)->shouldReturn(CUSTOM_STYLESHEET_DIRECTORY_URI);

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getCdnUri(CUSTOM_STYLESHEET_DIRECTORY_URI)->shouldReturn("$static_uri/" . CUSTOM_STATIC_STYLESHEET_DIRECTORY);
    }

    public function it_will_return_a_stylesheet_based_cdn_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');

        $this->getCdnUri()->shouldReturn(CUSTOM_STYLESHEET_DIRECTORY_URI);

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getCdnUri()->shouldReturn("$static_uri/" . CUSTOM_STATIC_STYLESHEET_DIRECTORY);
    }

    public function it_will_return_current_host(): void
    {
        $this->getCurrentHost()->shouldReturn(parse_url(SITE_URL, PHP_URL_HOST));
    }

    public function it_will_return_a_correct_plugin_path(): void
    {
        $this
            ->pluginPath('my-custom-plugin')
            ->shouldReturn(ABSPATH . 'wp-content/plugins/my-custom-plugin');
    }

    public function it_will_return_a_plugins_cdn_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');
        $pluginName = 'my-plugin';

        $this->getStaticPluginUri($pluginName)->shouldReturn(PLUGINS_URL . "/$pluginName");

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getStaticPluginUri($pluginName)->shouldReturn("$static_uri/wp-content/plugins/$pluginName");
    }

    public function it_will_return_a_themes_cdn_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');

        $this->getStaticThemeUri()->shouldReturn(CUSTOM_STYLESHEET_DIRECTORY_URI);

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getStaticThemeUri()->shouldReturn("$static_uri/" . CUSTOM_STATIC_STYLESHEET_DIRECTORY);
    }

    public function it_will_return_a_file_from_cdn_uri_if_in_production(): void
    {
        $static_uri = env('CF_STATIC');
        $file_path = 'dir/file.php';

        $this->getCdnFile($file_path)->shouldReturn(CUSTOM_STYLESHEET_DIRECTORY_URI . "/$file_path");

        Environment::setCurrent(Environment::PRODUCTION);

        $this->getCdnFile($file_path)->shouldReturn("$static_uri/" . CUSTOM_STATIC_STYLESHEET_DIRECTORY . "/$file_path");
    }

    public function it_returns_a_correct_child_theme_path(): void
    {
        $path = 'dir/file.php';
        $this->childThemePath($path)->shouldReturn(CUSTOM_STYLESHEET_DIRECTORY . "/$path");
    }

    public function it_returns_a_correct_perent_theme_path(): void
    {
        $path = 'dir/file.php';
        $this->themePath($path)->shouldReturn(CUSTOM_TEMPLATE_DIRECTORY . "/$path");
    }
}
